// Format: s.[servonum].[pos]. Ex: s.1.145 (servo 1 to 145)
// Be sure to set Serial Monitor to Carriage Return

#include <Messenger.h>             // Include Messenger object library
#include <Servo.h>

const int maxServos = 6;           // Total number of servos
const int offset = 4;              // Start with D4

// These are for reference and further expansion by you
const int base =       0;          // 60; 160 ccw, 0 cw
const int shoulder =   1;          // 130, 180 (full back); 120 (forward)
const int elbow =      2;          // 110, 150 fwd, 60 back (full back when shoulder fwd)
const int wristflex =  3;          // 20, 160 full back, 20 full down
const int wristrot =   4;          // 110, 10 ccw, 180 cw
const int gripper =    5;          // 95, 160 closed, 20 open

typedef struct{                    // Structure for holding values for each servo
  byte neutral;
  byte minPos;
  byte maxPos;
  byte delaySpeed;                 // Not used in this version
  byte curPos;
} ServoPos;

ServoPos servosPos[] = {           // Array of ServoPos structures (1 for each servo)
  { 60, 160, 0, 0, 0 },
  { 130, 180, 120, 0, 0 },
  { 110, 150, 60, 0, 0 },
  { 20, 160, 5, 0, 0 },
  { 110, 180, 10, 0, 0 },
  { 95, 160, 20, 0, 0 }
};

Servo servos[maxServos];              // Array of servo objects  
Messenger message = Messenger('.');   // Messenger object constructor

void setup() {
  Serial.begin(9600);
  for(int i=0; i<maxServos; i++) {            // 0 to 5
    servos[i].write(servosPos[i].neutral);    // Preset servos to neutral
    servos[i].attach(i+offset);               // Attach servo, starting at D4
    delay(1000);
  }
  message.attach(messageCompleted);   // Attach Messenger to messageCompleted function
}

void loop() {
  while (Serial.available()) {        // If bytes in Serial port
    message.process(Serial.read());   // Read message in Serial port
  }
  delay(100);
}

void messageCompleted() {             // Process if message starts with 's'
  if (message.checkString("s")) 
    writeServo();
}

void writeServo() {
  int servoNum = message.readInt();               // Get servo number
  if(servoNum >=0 && servoNum <= maxServos) {
    byte t = message.readInt();                   // Get servo position
    // Constrain values to max and min
    servosPos[servoNum].curPos = constrain(t, servosPos[servoNum].maxPos, servosPos[servoNum].minPos);
    servos[servoNum].write(servosPos[servoNum].curPos);  // Move to position
    Serial.print("Servo: ");                      // Print servo number and position
    Serial.print(servoNum, DEC);
    Serial.print(", Position: ");
    Serial.println(servosPos[servoNum].curPos, DEC);
  }
}
